local Addon = ...
local VERSION = 19

local loot_btns = {}
local hide_frames = {}
local mlevel = 15
local wchest = true

local function update()
	local backframe = _G["LootFinder_Scroll"]
	local frame = _G["LootFinderFrame"]
	for i = #loot_btns, #LootFinder.loot_list do -- loop to create new buttons, when lootcount > buttoncount
		local btn = LootFinder.UI.LootFinderButton(nil, backframe)
		loot_btns[#loot_btns + 1] = btn
        btn:SetID(i + 1)
		btn:SetSize(frame:GetWidth()-8,20)
		btn:SetPoint("TOPLEFT",4,-(i)*20)
        btn:SetScript("OnClick",function(self) 
            local item = LootFinder.loot_list[self:GetID()]
            if IsShiftKeyDown() then
                local index = 0
                if LootFinder.spec > 0 then
                    index = LootFinder.spec
                else
                    index = LootFinder.class
                end
                LootFinder.loot_list = LootFinderDB.FavLoot[index] or LootFinder.loot_list
                LootFinder:Find()
                update()
            else
                LootFinder.Favorite(item)  
                LootFinder:Find()
                update() 
            end

        end)
	end
	for i = 1, #loot_btns do -- hide all buttons for situation when buttoncoutn > lootcount
		loot_btns[i]:Hide()
	end
	for index,tbl in pairs(LootFinder.loot_list)  do -- show and set information (self,dungeon,itemlink)
		local btn = loot_btns[index]
		btn:SetSource(tbl.source)
		btn.boss = tbl.boss
		btn.dung = tbl.name
        btn:SetFavorite(tbl.itemlink)
		btn:SetIcon(tbl.icon)
		btn:SetDungeon(tbl.name)
		btn.link = tbl.itemlink
		btn:SetMainAtr(tbl.mainstat)
		btn:SetCrit(tbl.crit)
		btn:SetHaste(tbl.haste)
		btn:SetMastery(tbl.mastery)
		btn:SetVersality(tbl.versality)
		btn:Show()
	end
end

function LootFinder.createmyclasslist(btn)
    -- background for class buttons
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 4 + 20, btnsize * 3 + 20)
    -- class buttons with onclick script
    for i = 1, 12 do
        local button = LootFinder.UI.Button(nil, back)
        button:SetPoint("TOPLEFT",(i-1)%4*btnsize+10,-1*math.floor((i-1)/4)*btnsize-10)
        button:SetSize(btnsize,btnsize)
        button:SetID(i)
        local _, class, _ = GetClassInfo(i)
        button.texture:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES")
        local coords = CLASS_ICON_TCOORDS[class]
        button.texture:SetTexCoord(unpack(coords))

        button:SetScript("onclick",function(self)
            LootFinder.class = self:GetID()
            LootFinder.spec = 0
            -- specrefresh()
            btn.texture:SetTexture(self.texture:GetTexture())
            local _, class, _ = GetClassInfo(self:GetID())
            local coords = CLASS_ICON_TCOORDS[class]
            btn.texture:SetTexCoord(unpack(coords))
            back:Hide()
            LootFinder:Find()
        end)
    end
    hide_frames[#hide_frames + 1] = back
  --  back:SetScript("OnLeave",function(self) self:Hide() end)
    back:Hide()
    return back
end

function LootFinder.createslotlist(btn)
    -- background for spec buttons
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 4 + 20, btnsize * 4 + 20) -- change to setpoint
    local icons = {133070,133292,135061,133771,132751,132616,132939,132511,134584,132535,135317,134959,801523,133441,QUESTION_MARK_ICON}
    local slots = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,15}

    for i = 1, #icons do 
        local button = LootFinder.UI.Button(nil, back)
        button:SetPoint("TOPLEFT",(i-1)%4*btnsize+10,-1*math.floor((i-1)/4)*btnsize-10)
        button:SetSize(btnsize,btnsize)
        button.texture:SetTexture(icons[i])
        button:SetID(slots[i])
        button:SetScript("OnClick",function(self)
            LootFinder.slot = self:GetID()
            btn.texture:SetTexture(self.texture:GetTexture())
            back:Hide()
            LootFinder:Find()
        end)
    end
    hide_frames[#hide_frames + 1] = back
    --back:SetScript("OnLeave",function(self) self:Hide() end)
    back:Hide()
    return back
end
function LootFinder.createspeclist(btn)
    local spec_btn_list = {}
    -- background for spec buttons
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 3 + 20, btnsize * 2 + 20)
    for i = 1, 5 do
        local button = LootFinder.UI.Button(nil, back)
        button:SetPoint("TOPLEFT",(i-1)%3*btnsize+2,-1*math.floor((i-1)/3)*btnsize-2)
        button:SetSize(btnsize,btnsize)
        button:SetScript("OnClick",function(self)
            LootFinder.spec = self:GetID()
            btn.texture:SetTexture(self.texture:GetTexture())
            back:Hide()
            LootFinder:Find()
        end)
        spec_btn_list[i] = button
    end

    back:SetScript("OnShow",function(self) 
        local len = #LootFinder.class_spec[LootFinder.class] 
        if len == 0 then self:Hide() end -- hide if no class choosen
        local spec_ids = LootFinder.class_spec[LootFinder.class]
        if len == 2 then back:SetSize(btnsize * 3 + 10, btnsize * 1 + 10) else back:SetSize(btnsize * 3 + 10, btnsize * 2 + 10) end
        for i = 1, len do
           local _, _, _, icon, _, _ = GetSpecializationInfoByID(spec_ids[i])
           spec_btn_list[i]:SetID(spec_ids[i])
           spec_btn_list[i].texture:SetTexture(icon)
           spec_btn_list[i]:Show()
        end
        spec_btn_list[len+1]:SetID(0)
        spec_btn_list[len+1].texture:SetTexture(QUESTION_MARK_ICON)
        spec_btn_list[len+1]:Show()
        for i = len + 2, 5 do
            spec_btn_list[i]:Hide()
        end
    end)
    hide_frames[#hide_frames + 1] = back
    --back:SetScript("OnLeave",function(self) self:Hide() end)
    back:Hide()
    return back
end
-- instances
function LootFinder.createinstancelist(btn)

    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 4 + 20, btnsize * 4 + 20)

    local i = 1
    EJ_SelectTier(LootFinder.expansion)
    while EJ_GetInstanceByIndex(i, false) ~= nil do
        LootFinder.instances[i] = true
        local _, name, _, _, _, _, buttonImage2 = EJ_GetInstanceByIndex(i, false)
        local button = LootFinder.UI.Button(nil, back)
        button.name = name
        button:SetPoint("TOPLEFT",(i-1)%4*btnsize+10,-1*math.floor((i-1)/4)*btnsize-10)
        button:SetSize(btnsize,btnsize)
        button:SetID(i)
        button.texture:SetTexture(buttonImage2)
        button.find = true

        function button:enable(bool)
            if bool then
                LootFinder.instances[self:GetID()] = true
                self.texture:SetAlpha(1)
            else
                LootFinder.instances[self:GetID()] = false
                self.texture:SetAlpha(0.3)
            end
            self.find = bool
        end

        button:SetScript("OnClick",function(self)
            self:enable(not self.find)
            LootFinder:Find()
        end)

        button:SetScript("OnEnter",function(self)
            GameTooltip:Hide();
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:AddLine(self.name)
            GameTooltip:Show()
        end)

        button:SetScript("OnLeave",function()
            GameTooltip:Hide();
        end)

        i = i + 1
    end
    back:SetHeight(math.ceil((i-1)/4)*btnsize+20)

    hide_frames[#hide_frames + 1] = back
    --back:SetScript("OnLeave",function(self) self:Hide() end)
    back:Hide()
    return back
end
-- ilvl
function LootFinder.createmlevel(btn)
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 5 + 20, btnsize * 4 + 20)

    for i = 1, 15 do
        local button = LootFinder.UI.Button(nil, back)
        button:SetSize(btnsize, btnsize)
        button:SetPoint("TOPLEFT",(i-1)%5*btnsize+10,-1*math.floor((i-1)/5)*btnsize-10)

        button:SetText(i == 1 and 0 or i)
        button:SetID(i == 1 and 0 or i)

        button:SetScript("OnClick",function (self)
            local id = self:GetID()
            local chest, key = C_MythicPlus.GetRewardLevelForDifficultyLevel(max(1,id))
            if wchest then
                LootFinder.milvl = chest
            else
                LootFinder.milvl = key
            end
            LootFinder.mlevel = id
            btn:SetText(LootFinder.mlevel .. (wchest and "|Tinterface/worldmap/treasurechest_64.blp:20:20|t" or ""))
            LootFinder:Find()
        end)

    end
    local chesckbtn = CreateFrame("CheckButton", "LootFinderReward", back, "ChatConfigCheckButtonTemplate");
    chesckbtn:SetPoint("TOPLEFT", 10, -btnsize * 3 -10);
    LootFinderRewardText:SetText(MYTHIC_PLUS_WEEKLY_BEST);
    chesckbtn:SetChecked(true)
    chesckbtn:SetScript("OnClick",function(self)
        local checked = self:GetChecked()
        local chest, key = C_MythicPlus.GetRewardLevelForDifficultyLevel(max(1,mlevel))
        if checked == true then
            LootFinder.milvl = chest
        else
            LootFinder.milvl = key
        end
        wchest = checked
        btn:SetText(LootFinder.mlevel .. (wchest and "|Tinterface/worldmap/treasurechest_64.blp:20:20|t" or ""))
        LootFinder:Find()
    end);
    hide_frames[#hide_frames + 1] = back

    back:Hide()
    return back
end

-- stat choose (later replace this)

local function createfilters(btn)
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)

    local displayname = {
    --    SPEC_FRAME_PRIMARY_STAT_STRENGTH, -- comment out because useless
    --    SPEC_FRAME_PRIMARY_STAT_AGILITY, -- comment out because useless
    --    SPEC_FRAME_PRIMARY_STAT_INTELLECT, -- comment out because useless
        STAT_CRITICAL_STRIKE,
        STAT_HASTE,
        STAT_VERSATILITY,
        STAT_MASTERY
    }
    local values = {
    --    'ITEM_MOD_STRENGTH_SHORT', -- comment out because useless
    --    'ITEM_MOD_AGILITY_SHORT', -- comment out because useless
    --    'ITEM_MOD_INTELLECT_SHORT', -- comment out because useless
        'ITEM_MOD_CRIT_RATING_SHORT',
        'ITEM_MOD_HASTE_RATING_SHORT',
        'ITEM_MOD_VERSATILITY',
        'ITEM_MOD_MASTERY_RATING_SHORT'
    }
    back:SetSize(150,10 + #values * 20)
    for i = 1, #values do
        local checkbtn = CreateFrame("CheckButton", "LootFinder_StatChoose"..i, back, "ChatConfigCheckButtonTemplate")
        checkbtn:SetPoint("TOPLEFT", 5, -i*20+15)
        checkbtn.val = values[i]
        _G["LootFinder_StatChoose"..i.."Text"]:SetText(displayname[i])
        checkbtn:SetScript("OnClick",function(self)
            local checked = self:GetChecked()
            if checked == true then
                LootFinder.stats[self.val] = true
            else
                LootFinder.stats[self.val] = nil
            end
            LootFinder:Find()
        end)
    end
    hide_frames[#hide_frames + 1] = back
    back:Hide()
    return back
    --[[
        SPEC_FRAME_PRIMARY_STAT_STRENGTH    'ITEM_MOD_STRENGTH_SHORT'
        SPEC_FRAME_PRIMARY_STAT_AGILITY     'ITEM_MOD_AGILITY_SHORT'
        SPEC_FRAME_PRIMARY_STAT_INTELLECT   'ITEM_MOD_INTELLECT_SHORT'
        STAT_CRITICAL_STRIKE                'ITEM_MOD_CRIT_RATING_SHORT'
        STAT_HASTE                          'ITEM_MOD_HASTE_RATING_SHORT'
        STAT_VERSATILITY                    'ITEM_MOD_VERSATILITY'
        STAT_MASTERY                        'ITEM_MOD_MASTERY_RATING_SHORT'
    ]]
end

local function createraidlist(btn)
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    local btnsize = 30
    back:SetSize(btnsize * 4 + 20, 150)

    local i = 1
    EJ_SelectTier(LootFinder.expansion)
    while EJ_GetInstanceByIndex(i, true) ~= nil do
        LootFinder.raids[i] = true
        local _, name, _, _, _, _, buttonImage2 = EJ_GetInstanceByIndex(i, true)
        local button = LootFinder.UI.Button(nil, back)
        button.name = name
        button:SetPoint("TOPLEFT",(i-1)%4*btnsize+10,-1*math.floor((i-1)/4)*btnsize-10)
        button:SetSize(btnsize,btnsize)
        button:SetID(i)
        button.texture:SetTexture(buttonImage2)
        button.find = true

        function button:enable(bool)
            if bool then
                LootFinder.raids[self:GetID()] = true
                self.texture:SetAlpha(1)
            else
                LootFinder.raids[self:GetID()] = false
                self.texture:SetAlpha(0.3)
            end
            self.find = bool
        end

        button:SetScript("OnClick",function(self)
            self:enable(not self.find)
            LootFinder:Find()
        end)

        button:SetScript("OnEnter",function(self)
            GameTooltip:Hide();
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:AddLine(self.name)
            GameTooltip:Show()
        end)

        button:SetScript("OnLeave",function()
            GameTooltip:Hide();
        end)

        i = i + 1
    end
    local diff_names = {
        PLAYER_DIFFICULTY1, -- normal
        PLAYER_DIFFICULTY2, -- heroic
        PLAYER_DIFFICULTY6, -- mythic
        PLAYER_DIFFICULTY3  -- raid finder
    }
    local diff_ids = {
        14, -- normal
        15, -- heroic
        16, -- mythic
        17  -- raid finder
    }
    for i = 1, 4 do
        local checkbtn = CreateFrame("CheckButton", "LootFinder_RaidChoose"..i, back, "ChatConfigCheckButtonTemplate")
        checkbtn:SetPoint("TOPLEFT", 5, -(i+1) *20-20)
        checkbtn.val = diff_ids[i]
        _G["LootFinder_RaidChoose"..i.."Text"]:SetText(diff_names[i])
        checkbtn:SetScript("OnClick",function(self)
            for j = 1,4 do
                _G["LootFinder_RaidChoose"..j]:SetChecked(false)
            end
            self:SetChecked(true)
            LootFinder.raid_difficult = self.val

            LootFinder:Find()
        end)
    end
    table.wipe(diff_names)
    diff_names = nil
    table.wipe(diff_ids)
    diff_ids = nil

    hide_frames[#hide_frames + 1] = back
    --back:SetScript("OnLeave",function(self) self:Hide() end)
    back:Hide()
    return back
end

local function createpvplist(btn)
    local back = CreateFrame("Frame",nil,btn,"InsetFrameTemplate")
    back:SetPoint("BOTTOMLEFT",0,30)
    back:SetSize(120, 130)
    local nopvp = CreateFrame("CheckButton", "WeekKeys_PvPChoose0", back, "ChatConfigCheckButtonTemplate")
    nopvp:SetPoint("TOPLEFT", 5, -5)
    nopvp.val = 0
    nopvp:SetChecked(true)
    _G["WeekKeys_PvPChoose0Text"]:SetText(DISABLE)
    nopvp:SetScript("OnClick",function(self)
        for j = 0,5 do
            _G["WeekKeys_PvPChoose"..j]:SetChecked(false)
        end
        self:SetChecked(true)
        LootFinder.pvptier = self.val

        LootFinder:Find()
    end)
    local rating = {
        "0-1399",
        "1400-1599",
        "1600-1799",
        "1800-2099",
        "2100+"
    }
    for i = 0, 4 do
        local checkbtn = CreateFrame("CheckButton", "WeekKeys_PvPChoose"..(i+1), back, "ChatConfigCheckButtonTemplate")
        checkbtn:SetPoint("TOPLEFT", 5, -(i+1) *20-5)
        checkbtn.val = i+1
        checkbtn.tooltip = rating[i+1]
        _G["WeekKeys_PvPChoose"..(i+1).."Text"]:SetText(_G["PVP_RANK_".. i .."_NAME"])
        checkbtn:SetScript("OnClick",function(self)
            for j = 0,5 do
                _G["WeekKeys_PvPChoose"..j]:SetChecked(false)
            end
            self:SetChecked(true)
            LootFinder.pvptier = self.val
            LootFinder:Find()
        end)
    end
    hide_frames[#hide_frames + 1] = back
    back:Hide()
    return back
--[[
    local pvpnames = {
        PVP_RANK_0_NAME,
        PVP_RANK_1_NAME,
        PVP_RANK_2_NAME,
        PVP_RANK_3_NAME,
        PVP_RANK_4_NAME,
        PVP_RANK_5_NAME
    }--]]

end

--]]
--mylist = nil


function LootFinder.Create()
    local frame = CreateFrame('Frame', 'LootFinderFrame', UIParent,"BasicFrameTemplateWithInset")
    tinsert(UISpecialFrames, 'LootFinderFrame')
    frame:SetSize(500,500)
    LootFinderDB = LootFinderDB or {}
    LootFinderDB.Settings = LootFinderDB.Settings or {}
    if LootFinderDB.Settings["MainFramePos"] then
        local db = LootFinderDB.Settings["MainFramePos"]
        frame:SetPoint(db["point"],db["xOfs"],db["yOfs"])
    else
        frame:SetPoint("center")
    end

    frame.AddonName = frame:CreateFontString(nil , "BORDER", "GameFontNormal")
    frame.AddonName:SetPoint("top",0,-5)
    frame.AddonName:SetFormattedText("%s (v%d)",Addon, VERSION)
    frame.AddonName:SetTextColor(1,1,1)
    frame:SetMovable(true)
    frame:RegisterForDrag("LeftButton")

    frame:SetScript("OnMouseDown", function(self, button)
        if button == "LeftButton" and not self.isMoving then
            self.isMoving = true;
            self:StartMoving();
        end
    end)

    frame:SetScript("OnMouseUp", function(self, button)
        if button == "LeftButton" and self.isMoving then
            self:StopMovingOrSizing();
            self.isMoving = false;

            local point, _, _, xOfs, yOfs = frame:GetPoint()

            LootFinderDB.Settings["MainFramePos"] = {
                ["point"] = point,
                ["xOfs"] = xOfs,
                ["yOfs"] = yOfs,
            }
        end
    end)
    frame:SetScript("OnHide", function(self)
        if ( self.isMoving ) then
            self:StopMovingOrSizing();
            self.isMoving = false;
        end
    end)
    frame.ScrollFrame = CreateFrame("Scrollframe",nil , frame,"UIPanelScrollFrameTemplate")
    frame.ScrollFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -100);
    frame.ScrollFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5);
    frame.ScrollFrame:SetClipsChildren(true);

    frame.ScrollFrame.ScrollBar:ClearAllPoints();
    frame.ScrollFrame.ScrollBar:SetPoint("TOPLEFT", frame.ScrollFrame, "TOPRIGHT", -12, -18);
    frame.ScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", frame.ScrollFrame, "BOTTOMRIGHT", -7, 23  );

    frame.ScrollFrame:SetScript("OnMouseWheel", function(self, delta)
        local newValue = self:GetVerticalScroll() - (delta * 20);

        if (newValue < 0) then
            newValue = 0;
        elseif (newValue > self:GetVerticalScrollRange()) then
            newValue = self:GetVerticalScrollRange();
        end

        self:SetVerticalScroll(newValue);
    end);
-----------------------------------
----------
-----------------------------------

    local backframe = CreateFrame('Frame',"LootFinder_Scroll")
    frame.ScrollFrame:SetScrollChild(backframe)
    -- mylist = arrayOfElements[1]
    backframe:SetFrameStrata("BACKGROUND")
    backframe:SetSize(10,10)

    local class_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    class_label:SetSize(30, 20)
    class_label:SetPoint("TOPLEFT", 20, -30)
    class_label:SetText(CLASS)
    -- class btn
    local class_btn = LootFinder.UI.Button(nil, frame)
    class_btn:SetSize(30,30)
    class_btn:SetPoint("TopLeft",20,-50)
    class_btn.texture:SetTexture(QUESTION_MARK_ICON)
    class_btn:SetScript("OnClick",function(self) 
        self.showframe = self.showframe or LootFinder.createmyclasslist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    class_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(CLASS)
        GameTooltip:Show()
    end)

    class_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local spec_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    spec_label:SetSize(30, 20)
    spec_label:SetPoint("TOPLEFT", 60, -30)
    spec_label:SetText(SPECIALIZATION)
    -- spec btn
    local spec_btn = LootFinder.UI.Button(nil, frame)
    spec_btn:SetSize(30,30)
    spec_btn:SetPoint("Topleft",60,-50)
    spec_btn.texture:SetTexture(QUESTION_MARK_ICON)
    spec_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or LootFinder.createspeclist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    spec_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(SPECIALIZATION)
        GameTooltip:Show()
    end)

    spec_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local slot_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    slot_label:SetSize(30, 20)
    slot_label:SetPoint("TOPLEFT", 100, -30)
    slot_label:SetText(SLOT_ABBR)
    -- slot btn
    local slot_btn = LootFinder.UI.Button(nil, frame)
    slot_btn:SetSize(30,30)
    slot_btn:SetPoint("Topleft",100,-50)
    slot_btn.texture:SetTexture(QUESTION_MARK_ICON)
    slot_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or LootFinder.createslotlist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    slot_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(SLOT_ABBR)
        GameTooltip:Show()
    end)

    slot_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local instance_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    instance_label:SetSize(30, 20)
    instance_label:SetPoint("TOPLEFT", 140, -30)
    instance_label:SetText(INSTANCE)
    -- instances
    local instance_btn = LootFinder.UI.Button(nil, frame)
    instance_btn:SetSize(30,30)
    instance_btn:SetPoint("Topleft",140,-50)
    instance_btn.texture:SetTexture('interface/minimap/objecticonsatlas.blp')
    instance_btn.texture:SetTexCoord(0.1728515625,0.1943359375,0.912109375,0.955078125)
    instance_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or LootFinder.createinstancelist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    instance_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(INSTANCE)
        GameTooltip:Show()
    end)

    instance_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local raid_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    raid_label:SetSize(30, 20)
    raid_label:SetPoint("TOPLEFT", 180, -30)
    raid_label:SetText(RAID)
    local raid_btn = LootFinder.UI.Button(nil, frame)
    raid_btn:SetSize(30,30)
    raid_btn:SetPoint("Topleft",180,-50)
    raid_btn.texture:SetTexture('interface/minimap/objecticonsatlas.blp')
    raid_btn.texture:SetTexCoord(0.5009765625,0.5224609375,0.294921875,0.337890625)
    raid_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or createraidlist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    raid_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(RAID)
        GameTooltip:Show()
    end)
    raid_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local pvp_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    pvp_label:SetSize(30, 20)
    pvp_label:SetPoint("TOPLEFT", 220, -30)
    pvp_label:SetText(PLAYER_V_PLAYER)

    --PLAYER_V_PLAYER
    local pvp_btn = LootFinder.UI.Button(nil, frame)
    pvp_btn:SetSize(30,30)
    pvp_btn:SetPoint("Topleft",220,-50)
    pvp_btn.texture:SetTexture('Interface/TalentFrame/TalentFrameAtlas.blp')
    pvp_btn.texture:SetTexCoord(0.75390625,0.93359375,0.1015625,0.1435546875)
    pvp_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or createpvplist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    pvp_btn:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(PLAYER_V_PLAYER)
        GameTooltip:Show()
    end)

    pvp_btn:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    

    --[[
    local pvp_btn = WeekKeys.UI.Button(nil, WeekKeys.WeekFrame)
    pvp_btn:SetSize(30,30)
    pvp_btn:SetPoint("Topleft",180,-30)
    pvp_btn.texture:SetTexture('????')
    pvp_btn.texture:SetTexCoord(some_text_coords_if_need)
    pvp_btn:SetScript("OnClick",function(self)
        self.showframe = self.showframe or createpvplist(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    pvp_btn:Hide()
    arrayOfElements[#arrayOfElements + 1] = pvp_btn
    --]]
	

    local key_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    key_label:SetSize(30, 20)
    key_label:SetPoint("TOPLEFT", 260, -30)
    key_label:SetText(LEVEL)
    -- keylevel
    local keylevel = LootFinder.UI.Button(nil, frame)
    keylevel:SetSize(30,30)
    keylevel:SetPoint("Topleft",260,-50)
    keylevel:SetText(15 .. "|Tinterface/worldmap/treasurechest_64.blp:20:20|t")
    keylevel:SetScript("OnClick",function(self)
        self.showframe = self.showframe or LootFinder.createmlevel(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    keylevel:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(LEVEL)
        GameTooltip:Show()
    end)

    keylevel:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)

    local stat_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    stat_label:SetSize(30, 20)
    stat_label:SetPoint("TOPLEFT", 300, -30)
    stat_label:SetText(STAT_CATEGORY_ENHANCEMENTS)
    -- filters
    local statfilter = LootFinder.UI.Button(nil, frame)
    statfilter:SetSize(30,30)
    statfilter:SetPoint("Topleft",300,-50)
    statfilter:SetText("???")
    statfilter:SetScript("OnClick",function(self)
        self.showframe = self.showframe or createfilters(self)
        if self.showframe:IsShown() then
            return self.showframe:Hide()
        end
        for _,v in pairs(hide_frames) do v:Hide() end
        self.showframe:Show()
    end)
    statfilter:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(STAT_CATEGORY_ENHANCEMENTS)
        GameTooltip:Show()
    end)

    statfilter:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)


    local fav_label = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    fav_label:SetSize(30, 20)
    fav_label:SetPoint("TOPLEFT", 340, -30)
    fav_label:SetText(AUCTION_HOUSE_FAVORITES_SEARCH_TOOLTIP_TITLE)

    local fav = LootFinder.UI.Button(nil, frame)
    fav:SetSize(30,30)
    fav:SetPoint("Topleft",340,-50)
    fav.texture:SetTexture('Interface/AuctionFrame/AuctionHouse.blp')
    fav.texture:SetTexCoord(0.9580078125,0.9833984375,0.591796875,0.642578125)
    fav:SetScript("OnClick",function(self)
        if LootFinder.FavLoot then
            self.texture:SetTexCoord(0.9580078125,0.9833984375,0.591796875,0.642578125)
            LootFinder.FavLoot = not LootFinder.FavLoot
        else
            self.texture:SetTexCoord(0.9306640625,0.9560546875,0.591796875,0.642578125)
            LootFinder.FavLoot = not LootFinder.FavLoot
        end
        LootFinder:Find()
        update()
    end)

    fav:SetScript("OnEnter",function(self)
        GameTooltip:Hide();
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:AddLine(AUCTION_HOUSE_FAVORITES_SEARCH_TOOLTIP_TITLE)
        GameTooltip:Show()
    end)

    fav:SetScript("OnLeave",function()
        GameTooltip:Hide();
    end)
	
	local label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",25,-80)
    label:SetSize(200,20)
    label:SetText(DUNGEONS)
    local fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(200,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("name") update() end)
	
	label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",230,-80)
    label:SetSize(50,20)
    label:SetText(SPEC_FRAME_PRIMARY_STAT)
    fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(50,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("mainstat") update() end)
	
    label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",280,-80)
    label:SetSize(50,20)
    label:SetText(SPELL_CRIT_CHANCE)
    fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(50,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("crit") update() end)

    label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",330,-80)
    label:SetSize(50,20)
    label:SetText(STAT_HASTE)
    fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(50,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("haste") update() end)

    label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",380,-80)
    label:SetSize(50,20)
    label:SetText(STAT_MASTERY)
    fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(50,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("mastery") update() end)

    label = LootFinder.UI.Button(nil, frame)
    label:SetPoint("TOPLEFT",430,-80)
    label:SetSize(50,20)
    label:SetText(STAT_VERSATILITY)
    fontstr = label:GetFontString()
    label:SetFontString(fontstr)
    fontstr:SetSize(50,20)
    label:SetScript("OnClick", function() LootFinder.SortBy("versality") update() end)

--[[
    --]]
    -- result printing
    hooksecurefunc(LootFinder, "Find", update)
    frame:Hide()
    return frame
end

