LootFinder.UI = {}


function LootFinder.UI.Button(name, parent)
    local button = CreateFrame('Button', name, parent)

    local txt = button:CreateTexture('textureName', 'BACKGROUND')
    txt:SetPoint('TOPLEFT', 2, 0)
    txt:SetPoint('bottomright', -2, 0)
    txt:SetColorTexture(1,1,1,0.2)
    button:SetHighlightTexture(txt)

    button.texture = button:CreateTexture('textureName', 'CENTER')
    button.texture:SetPoint('TOPLEFT', 2, 0)
    button.texture:SetPoint('bottomright', -2, 0)
    button.texture:SetColorTexture(0.2, 0.2, 0.2, 0)

    button:DisableDrawLayer("BORDER")
    button:SetNormalFontObject('GameFontNormal')

    if parent and parent:GetObjectType() == "Frame" then
        button:SetSize(parent:GetWidth()-4, 20)
        button:SetPoint("TOPLEFT",0,-parent:GetHeight())
    elseif parent and parent:GetObjectType() == "Button" then
        button:SetSize(parent:GetSize())
        button:SetPoint("TOPLEFT",0,-parent:GetHeight())
    end

    return button
end

function LootFinder.UI.LootFinderButton(name,parent)
    local btn = LootFinder.UI.Button(name, parent)

     -- favorite icon

     btn.favorite = btn:CreateTexture('textureName', 'CENTER')
     btn.favorite:SetPoint("LEFT",5,0)
     btn.favorite:SetSize(18,18)
     btn.favorite:SetTexture('Interface/AuctionFrame/AuctionHouse')
     btn.SetFavorite = function(self, item)
         if LootFinder.IsFavorite(item) then
             -- turn on
             self.favorite:SetTexCoord(0.9306640625,0.9560546875,0.591796875,0.642578125)
         else
            -- turn off
            self.favorite:SetTexCoord(0.9580078125,0.9833984375,0.591796875,0.642578125)
         end
     end

    btn.source = btn:CreateTexture('textureName', 'CENTER')
    btn.source:SetPoint("LEFT",25,0)
    btn.source:SetSize(20,20)
    ---Set Loot texture
    ---@param self Button
    ---@param source string
    btn.SetSource = function (self,source)
        if source == "pvp" then
            self.source:SetTexture('Interface/TalentFrame/TalentFrameAtlas.blp')
            self.source:SetTexCoord(0.75390625,0.93359375,0.1015625,0.1435546875)
        elseif source == "raid" then
            self.source:SetTexture('interface/minimap/objecticonsatlas.blp')
            self.source:SetTexCoord(0.5009765625,0.5224609375,0.294921875,0.337890625)
        elseif source == "instance" then
            self.source:SetTexture('interface/minimap/objecticonsatlas.blp')
            self.source:SetTexCoord(0.1728515625,0.1943359375,0.912109375,0.955078125)
        else
            self.source:SetTexture()
        end
    end

    -- icon
    btn.icon = btn:CreateTexture('textureName', 'CENTER')
    btn.icon:SetPoint("LEFT",45,0)
    btn.icon:SetSize(18,18)
    ---Set Loot texture
    ---@param self Button
    ---@param iconID integer
    btn.SetIcon = function (self,iconID)
        self.icon:SetTexture(iconID)
    end

    -- dungeon
    btn.instance = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.instance:SetPoint("LEFT",65,0)
    btn.instance:SetSize(160,20)
    btn.instance:SetJustifyH("LEFT")
    ---Set dungeon name
    ---@param self Button
    ---@param dungeon string name of dungeon
    btn.SetDungeon = function (self, dungeon)
        self.instance:SetText(dungeon or "")
    end

    -- main atr
    btn.mainatr = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.mainatr:SetPoint("LEFT",230,0)
    btn.mainatr:SetSize(50,20)
    ---Set Main atr value
    ---@param self Button
    ---@param mainatr integer str/agi/int value
    btn.SetMainAtr = function (self, mainatr)
        self.mainatr:SetText(mainatr or 0)
    end

    -- crit
    btn.crit = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.crit:SetPoint("LEFT",280,0)
    btn.crit:SetSize(50,20)
    ---Set Crit value
    ---@param self Button
    ---@param crit integer crit value
    btn.SetCrit = function (self, crit)
        self.crit:SetText(crit or 0)
    end

    -- haste
    btn.haste = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.haste:SetPoint("LEFT",330,0)
    btn.haste:SetSize(50,20)
    ---Set haste value
    ---@param self Button
    ---@param haste integer haste value
    btn.SetHaste = function (self, haste)
        self.haste:SetText(haste or 0)
    end

    -- mastery
    btn.mastery = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.mastery:SetPoint("LEFT",380,0)
    btn.mastery:SetSize(50,20)
    ---Set mastery value
    ---@param self Button
    ---@param mastery integer mastery value
    btn.SetMastery = function (self, mastery)
        self.mastery:SetText(mastery or 0)
    end

    -- vers
    btn.versality = btn:CreateFontString(nil , "ARTWORK", "GameFontNormal")
    btn.versality:SetPoint("LEFT",430,0)
    btn.versality:SetSize(50,20)
    ---Set versality value
    ---@param self Button
    ---@param versality integer versality value
    btn.SetVersality = function (self, versality)
        self.versality:SetText(versality or 0)
    end

    btn:SetScript("OnEnter",function(self)
        self:SetDungeon(self.boss)
        if not self.link then return end
        GameTooltip:Hide()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:SetHyperlink(self.link)
        GameTooltip:Show()
    end)
    btn:SetScript("OnLeave",function(self)
        self:SetDungeon(self.dung)
        GameTooltip:Hide()
    end)

    return btn
end

